#!/usr/bin/perl -w
# $Header: /usr/groups/printing/CUPS/local/RCS/lprwrapper,v 1.26 2014/09/03 19:48:21 pb22 Exp $
# Take "Traditional" lpr arguments and translate them into format suitable for CUPS lpr

#--------------------customisation variables-----------------------------
use Env;
use Getopt::Long;
$Getopt::Long::ignorecase=0;

$ENV{'PATH'} = "/usr/bin:/usr/sbin:$ENV{'PATH'}";
# Ubuntu uses:	dpkg-divert --package cl-lpr.cl --divert /usr/bin/lpr.real --rename /usr/bin/lpr
#		dpkg-divert --package cl-lpr.cl --remove                   --rename /usr/bin/lpr
@lprcups = ( "/usr/bin/lpr.real", "/usr/bin/lpr.cups", "/usr/bin/lpr");
# Is special case still needed for SUSE?
@lprcups = ( "/usr/bin/lpr") if -e "/etc/SuSE-release";
#$lpoptions="lpoptions";

# Take care not to re-call self. Set env var and compare paths
die("$0: command appears to be looping as \$IN_CL_LPRWRAPPER is set\n") if $ENV{'IN_CL_LPRWRAPPER'} ;
$ENV{'IN_CL_LPRWRAPPER'} = 1;
# Try to find a back end command to use
foreach $cmd (@lprcups) { $lprcups = $cmd, last if -r $cmd  && $cmd ne $0}
die("$0: no suitable lpr command found in @lprcups\n") unless $lprcups;

#---------------------- process arguments--------------------------------------

# Most convenient way to deal with options is to use GetOptions. However that expects arguments
# in the form -X arg, and not -Xarg as has been the case with lpr in the past. Thus we need to
# preprocess the arguments list into this format first. Also need to deal with special cases
# (eg -paper is a valid argument, we don't want it interpreted as -p aper, but -p37 should
# become -p 37. Likewise -transparency and -tumble).
# Also GetOptions doesn't like the '#' option, so turn it into a 'K', and then back
# into a '#' later
# Curiously, GetOptions can't tell you the difference between an absent flag and one with an
# optional argument, which causes problems with -p, so flag it if we see it here.
# Incidentally -transparancy is valid as several people complained that they could never spell
# -transparency correctly, and in the old lpr it wasn't easy to get feedback on incorrect options !

my $storeargs = "";
my $opt;
my $nextopt;
my @optchars;
my $pflag = 0;
my $topush1="";
my $topush2="";
my @ARGV2;
my @files;

# May (or may not) be set by GetOptions
$opt_staple="";
$opt_T = "";

for( $i = 0; $i < @ARGV; ++$i ){
  $storeargs = join " ",$storeargs,$ARGV[$i];
  $opt = $ARGV[$i];
  push @ARGV2,$opt;
  if ($opt =~ /^-staple/){
    $nextopt = $ARGV[$i+1];
    if ($nextopt && ($nextopt eq "1diag" || $nextopt eq "1" || $nextopt eq "2" || $nextopt eq "3" || $nextopt eq "6")){next;}
#have to assume argument is missing, it could be entirely bogus
    else{push @ARGV2,"default";}
  }
  if ( $opt !~ /^-paper/ && $opt !~ /^-transparency/ && $opt !~ /^-transparancy/ && $opt !~ /^-tumble/){
    @optchars = split //, $opt;
    if ( $optchars[0] eq "-" ){
      if ( $optchars[1] eq "#" ){ $optchars[1] = "K";}
      $x = $optchars[1];
      if ( $x eq "P" || $x eq "H" ||$x eq "T" || $x eq "p" || $x eq "c" || $x eq "i" || $x eq "m" || $x eq "t" || $x eq "h" || $x eq "o" || $x eq "K" ) {
	if( $x eq "p" ){$pflag = 1;}
        $topush1 = $optchars[0].$optchars[1];
        $topush2="";
        for( $p = 2; $p < @optchars; $p++ ){  $topush2 = join "",$topush2,$optchars[$p]; }
        pop @ARGV2;
        if( $topush2 ne "" ) {push @ARGV2, $topush1,$topush2;}
        else { push @ARGV2, $topush1; }
      }
    }
  }
}
@ARGV = @ARGV2;

my $result=&GetOptions("P=s","T=s","simplex|single","staple=s","f","tumble","transparency|transparancy|slides","paper","M","b","d","l","n","p:s","r","s","c=s","i=i","m=i","t=i","h=s","o=s","K=s","debug","noprint","a3","A3","a2","A2","a1","A1","a0","A0","u","w","H=s","J=s");

if( $result == 0 ) {&Usage;}

#Hope that anything left in @ARGV is a filename, if none it will use stdin
@files = @ARGV;

#-----------------------------------------------------------------------
# debug statements
if($opt_debug){
  print "lpr command is = $lprcups ($0)\n";
  if( $opt_P ){print "Printer = $opt_P \n";}
  if( $opt_H ){print "Server name = $opt_H \n";}
  if( $opt_J ){print "Job name = $opt_J \n";}
  if( $opt_T ){print "Type = $opt_T \n";}
  if( $opt_c ){print "c option = $opt_c \n"; }
  if( $opt_o ){ print "o option = $opt_o \n"; }
  if( $opt_h ){print "h option = $opt_h \n"; }
  if( $opt_simplex ){ print "simplex true \n"; }
  if( $opt_staple ){ print "staple type $opt_staple\n"; }
  if( $opt_tumble ){ print "tumble true \n"; }
  if( $opt_transparency ){ print "transparency true \n"; }
  #if( $opt_paper ){ print "paper true \n"; }
  if( $opt_M ){ print "M true \n"; }
  if( $opt_b ){ print "b true \n"; }
  if( $opt_d ){ print "d true \n"; }
  #if( $opt_f ){ print "f true \n"; }
  if( $opt_l ){ print "l true \n"; }
  if( $opt_n ){ print "n true \n"; }
  #if( $opt_s ){ print "s true \n"; }
  if( $opt_r ){ print "r true \n"; }
  #if( $opt_w ){ print "w true \n"; }
  if( $opt_a3 ){ print "a3 true \n"; }
  if( $opt_a2 ){ print "a2 true \n"; }
  if( $opt_a1 ){ print "a1 true \n"; }
  if( $opt_a0 ){ print "a0 true \n"; }
  if( $opt_A3 ){ print "A3 true \n"; }
  if( $opt_A2 ){ print "A2 true \n"; }
  if( $opt_A1 ){ print "A1 true \n"; }
  if( $opt_A0 ){ print "A0 true \n"; }
  if( $opt_t ){ print "t = $opt_t \n"; }
  if( $opt_u ){ print "u = $opt_u \n"; }
  if( $opt_i ){ print "i = $opt_i \n"; }
  if( $opt_m ){ print "m = $opt_m \n"; }
  if( $opt_K ){ print "K = $opt_K \n"; }
  if( $opt_p ){ print "p = $opt_p \n"; }
  if( !$opt_p && $pflag == 1){ print "p true \n"; } #is this used ?
}
#-----------------------------------------------------------------------
# Sort out which printer it is.
my $printer = "";

if( $opt_P ){
    $printer=$opt_P;
}else{
#we haven't been given a printer
  if( $PRINTER ne "" ){$printer=$PRINTER;}
}

if( $printer eq "" ){ die "Error: Unable to determine printer, please specify with -P flag\n";}
print "printer is $printer \n" if $opt_debug;

#-----------------------------------------------------------------------
# assemble command string

my $opts="";
my $cpi = 10;
my $pageleft=0;
my $pageright=0;
my $pagetop=0;
my $pagebottom=0;
my $margin=0;

#first of all, say we're using the wrapper
if( $opt_u ) {$opts="$opts";} #want utf8, at the moment do this by NOT using the wrapper
else {$opts="$opts -o lprwrapper ";}

# Types. Nothing special needed if ps, dvi or html as these are autorecognised and
#handled by filters.
if( $opt_T eq "compact" ) {$opts="$opts -o  a2psfont=compact";}
elsif( $opt_T eq "ljet" ) { $opts="$opts -o raw  ";}


#paper sizes
if($printer eq "redwood" || $printer eq "gum"){
  if( $opt_a3 || $opt_A3 ){$opts="$opts -o HPAutoScaling=A3 ";}
  if( $opt_a2 || $opt_A2 ){$opts="$opts -o HPAutoScaling=A2 ";}
  if( $opt_a1 || $opt_A1 ){$opts="$opts -o HPAutoScaling=A1 ";}
  if( $opt_a0 || $opt_A0 ){$opts="$opts -o HPAutoScaling=A0 ";}
}else{
#Kyocera 9500 series have an a3 tray (but different models need different options!), 
#Oki 9650 can have too
  if( $opt_a3 || $opt_A3 ){
      if ( $printer eq "sycamore" ){$opts="$opts -o PageSize=A3";  }
      else{
	  if($printer eq "willow" || $printer eq "poplar" || $printer eq "palm"){$opts="$opts -o InputSlot=PF700B -o PageSize=A3";}
	  else{
	      $opts="$opts -o media=A3 -o InputSlot=Tray2 -o PageSize=A3 -o PageRegion=A3";
	  }
      }
  }
}
#a4
if(!($opt_a3 || $opt_A3 || $opt_a2 || $opt_A2 || $opt_a1 || $opt_A1 || $opt_a0 || $opt_A0)){
    print "It's a4\n" if $opt_debug;
#    $opt_a4 = true;
    $pageleft=36;
    $pageright=36;
    $pagetop=36;
    $pagebottom=36;
#    $opts="$opts -o media=A4 ";
}

#stapling
if( $printer eq "palm" || $printer eq "poplar" || $printer eq "willow"){
  if( $opt_staple eq "1diag" || $opt_staple eq "1" || $opt_staple eq "2" || $opt_staple eq "3" || $opt_staple eq "6" || $opt_staple eq "default"){$opts = "$opts -o KCStaple=Upperleft ";}
}
if( $printer eq "mulberry"){
  if( $opt_staple eq "1diag" || $opt_staple eq "1" || $opt_staple eq "2" || $opt_staple eq "3" || $opt_staple eq "6" || $opt_staple eq "default"){$opts = "$opts -o StapleLocation=UpperLeft ";}
}

#transparency
if( $opt_transparency ){
  if ( $printer eq "maple" || $printer eq "birch" || $printer eq "ash" || $printer eq "beech"){
$opts="$opts -o InputSlot=PF60A  -o Duplex=None ";}
  else {$opts="$opts -o media=Transparency -o sides=one-sided ";}
}

#simplex
if( $opt_simplex ){$opts="$opts -o sides=one-sided  ";}
else{
    if( $opt_tumble ){$opts="$opts -o sides=two-sided-short-edge  ";}
#    else{$opts="$opts -o sides=two-sided-long-edge  ";}
}

#landscape: this can either be done in a2pswrapper or subsequent pstops stage.
#choose to do it in a2pswrapper so we get banners right.
if( $opt_l ){$opts="$opts -o a2pslandscape ";}

#Manual feed: have to find a neater way of doing this...
if( $opt_M ){
    if( $printer eq "palm" || $printer eq "cypress" || $printer eq "dogwood" || $printer eq "ebony" || $printer eq "fir" || $printer eq "larch" || $printer eq "lime" || $printer eq "pine" || $printer eq "royaloak" || $printer eq "sycamore" ){
$opts="$opts -o ManualFeed=True  ";}
    else{
#ricoh
	    if( $printer eq "acacia" || $printer eq "alder" || $printer eq "aspen" || $printer eq "elm" || $printer eq "ginkgo" || $printer eq "hickory" || $printer eq "hornbeam" || $printer eq "linden" || $printer eq "oak" || $printer eq "pear" || $printer eq "rowan" || $printer eq "teak" || $printer eq "walnut" || $printer eq "yew" ){
	   $opts="$opts -o InputSlot=MultiTray  ";
	    }
	    else{
#ie kyocera. Note has to be MF1, not MF as MF1=MF on all of them except 9520s,
#which only have MF1
	    $opts="$opts -o InputSlot=MF1  ";
	    }
    }
}

#mopies (I think we need to turn collating on by default)
if($opt_K && $opt_K != 0 ){$opts = "$opts -#".$opt_K." -o Collate=True ";}

#n-up
if($opt_m && $opt_m != 0 ){
  $opts="$opts -o number-up=".$opt_m." ";
  if( $opt_d ){$opts="$opts -o page-border=none ";}
  else{$opts="$opts -o page-border=double ";}
}

#lines per page
if($opt_i && $opt_i != 0 ){
    $opts="$opts -o linesperpage=".$opt_i." ";
}

#margins - need to set these every job because of CUPS misalignment misfeature
#send the raw option to the a2ps wrapper
if($opt_t && $opt_t != 0 ){
    $charsize = int 72 / $cpi;
    $leftmargin = int $opt_t * $charsize;
    $pageleft += $leftmargin;
    $margin = $opt_t;
}
    $opts="$opts -o page-left=".$pageleft." -o page-right=".$pageright." -o page-top=".$pagetop." -o page-bottom=".$pagebottom." -o a2psmargin=".$margin." ";


#columns
if($opt_c && $opt_c ne "" ){
  ($col,$row) = split(/x/, $opt_c);
  if( $row == 0 ){ $row=1;}
  $opts="$opts -o a2pscolumns=".$col." -o a2psrows=".$row." ";
}

#page ranges
if($opt_p && $opt_p ne "" ){
  $opts = "$opts -o page-ranges=".$opt_p." ";
  print "p range $opt_p\n" if $opt_debug;
}

#banner
if( !$opt_b ){$opts = "$opts -o prettyprint ";}

#line numbers
if( $opt_n ){$opts = "$opts -o line-numbers ";}

#job name
if( $opt_J ){$opts = "$opts -J \"".$opt_J."\" ";}

#other options
if($opt_o && $opt_o ne "" ){$opts="$opts -o ".$opt_o." ";}

if( $opt_r ){$opts = "$opts -r ";}

#--------------------output commands-----------------------------------

if($opt_H && $opt_H ne "" ){$cmd="$lprcups -P $printer -H $opt_H $opts ";}
else{$cmd="$lprcups -P $printer $opts ";}

if($#files >= 0 && $files[0] ne "") {
  for( $p = 0; $p < @files; $p++ ){
    print "file name $p is $files[$p]\n" if $opt_debug;
    $_ = $files[$p];
    s/\s/\\ /g;
    my $fullcmd = $cmd.$_;
    print "fullcmd is $fullcmd \n" if $opt_debug;
    if(!$opt_noprint){$output = `$fullcmd`;}
  }
}else{
#stdin presumably
  print "cmd is $cmd \n" if $opt_debug;
  if(!$opt_noprint){$output = `$cmd`;}
}
print "output is $output \n" if ($opt_debug && !$opt_noprint);
print "rc is $? \n" if $opt_debug;
exit( ($?) >> 8 | ($?));

#-----------------------------------------------------------------------
sub Usage {
print "lpr [-P printername | -Pprintername] [-T type | -Ttype] [-single|-simplex] [-tumble] [-transparency|-transparancy|-slide|-slides] [-paper]  [-staple] [-f] [-a3 | -A3] [-a2 | -A2] [-a1 | -A1] [-a0 | -A0] [-M] [-b] [-d] [-l] [-n] [-p num[:num]] [-c num] [-i num] [-m num] [-t num] [-h name] [-r] [-w] [-o option_string] [-debug] [-# num_copies] [ -H servername] [filename] \n";
exit 0;
}

#routine to extract info from lpoptions: might be useful one day but not used now
sub lpopts {
    $optcmd="lpoptions -p$printer -l";
    print "lpoptions string is $optcmd \n" if $opt_debug;
    @optoutput = `$optcmd`;
    foreach $opt (@optoutput) { print "optoutput is $opt \n" if $opt_debug;}
}
